@@ -3,7 +3,7 @@
 #include "image.h"
 
 static const nv_image_t rotation_orientation_horiz_image = {
-  120, 90, 3,
+  120, 90, 3, 0,
   "\377\0\0\0\363\0\0\0\35\7\15\20V\220\272t\277\366v\300\365x\301\365y\301"
   "\365{\302\365~\303\364\200\303\364\200\304\364\202\305\364\205\305\363\206"
   "\306\363\210\307\363\212\307\363\214\310\362\215\310\362\217\311\362\220"
